@@ -1,16 +1,16 @@
-# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -66,10 +66,10 @@ from .constants import (
     ServerCmd,
     ServerFlag,
     flag_is_set,
+    raise_warning_against_deprecated_cursor_class,
 )
 from .conversion import MySQLConverter
 from .cursor import (
-    CursorBase,
     MySQLCursor,
     MySQLCursorBuffered,
     MySQLCursorBufferedDict,
@@ -109,7 +109,14 @@ from .types import (
     StatsPacketType,
     StrOrBytes,
 )
-from .utils import get_platform, int1store, int4store, lc_int
+from .utils import (
+    get_platform,
+    int1store,
+    int4store,
+    lc_int,
+    warn_ciphersuites_deprecated,
+    warn_tls_version_deprecated,
+)
 
 if TYPE_CHECKING:
     from .abstracts import CMySQLPrepStmt
@@ -130,7 +137,6 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._converter_class: Type[MySQLConverter] = MySQLConverter
 
         self._client_flags: int = ClientFlag.get_default()
-        self._charset_id: int = 45
         self._sql_mode: Optional[str] = None
         self._time_zone: Optional[str] = None
         self._autocommit: bool = False
@@ -216,11 +222,17 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._character_set.set_mysql_version(self._server_version)
 
         if not handshake["capabilities"] & ClientFlag.SSL:
-            if self._auth_plugin == "mysql_clear_password" and not self.is_secure:
-                raise InterfaceError(
-                    "Clear password authentication is not supported over "
-                    "insecure channels"
-                )
+            if not self.is_secure:
+                if self._auth_plugin == "mysql_clear_password":
+                    raise InterfaceError(
+                        "Clear password authentication is not supported over "
+                        "insecure channels"
+                    )
+                if self._auth_plugin == "authentication_openid_connect_client":
+                    raise InterfaceError(
+                        "OpenID Connect authentication is not supported over "
+                        "insecure channels"
+                    )
             if self._ssl.get("verify_cert"):
                 raise InterfaceError(
                     "SSL is required but the server doesn't support it",
@@ -248,7 +260,6 @@ class MySQLConnection(MySQLConnectionAbstract):
         password: Optional[str] = None,
         database: Optional[str] = None,
         client_flags: int = 0,
-        charset: int = 45,
         ssl_options: Optional[Dict[str, Optional[Union[str, bool, List[str]]]]] = None,
         conn_attrs: Optional[Dict[str, str]] = None,
     ) -> bool:
@@ -284,11 +295,22 @@ class MySQLConnection(MySQLConnectionAbstract):
                 self._socket,
                 self.server_host,
                 ssl_options,
-                charset=charset,
+                charset=self._charset_id,
                 client_flags=client_flags,
             )
             self._ssl_active = True
 
+        # Add the custom configurations required by specific auth plugins
+        self._authenticator.update_plugin_config(
+            config={
+                "krb_service_principal": self._krb_service_principal,
+                "oci_config_file": self._oci_config_file,
+                "oci_config_profile": self._oci_config_profile,
+                "webauthn_callback": self._webauthn_callback,
+                "openid_token_file": self._openid_token_file,
+            }
+        )
+
         ok_pkt = self._authenticator.authenticate(
             sock=self._socket,
             handshake=self._handshake,
@@ -297,16 +319,11 @@ class MySQLConnection(MySQLConnectionAbstract):
             password2=self._password2,
             password3=self._password3,
             database=database,
-            charset=charset,
+            charset=self._charset_id,
             client_flags=client_flags,
             auth_plugin=self._auth_plugin,
             auth_plugin_class=self._auth_plugin_class,
             conn_attrs=conn_attrs,
-            krb_service_principal=self._krb_service_principal,
-            oci_config_file=self._oci_config_file,
-            oci_config_profile=self._oci_config_profile,
-            webauthn_callback=self._webauthn_callback,
-            fido_callback=self._fido_callback,
         )
         self._handle_ok(ok_pkt)
 
@@ -361,7 +378,6 @@ class MySQLConnection(MySQLConnectionAbstract):
                 self._password,
                 self._database,
                 self._client_flags,
-                self._charset_id,
                 self._ssl,
                 self._conn_attrs,
             )
@@ -379,18 +395,18 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         if (
             not self._ssl_disabled
-            and hasattr(self._socket.sock, "version")
-            and callable(self._socket.sock.version)
+            and hasattr(self._socket.sock, "cipher")
+            and callable(self._socket.sock.cipher)
         ):
-            # Raise a deprecation warning if TLSv1 or TLSv1.1 is being used
-            tls_version = self._socket.sock.version()
-            if tls_version in ("TLSv1", "TLSv1.1"):
-                warn_msg = (
-                    f"This connection is using {tls_version} which is now "
-                    "deprecated and will be removed in a future release of "
-                    "MySQL Connector/Python"
-                )
-                warnings.warn(warn_msg, DeprecationWarning)
+            # Raise a deprecation warning if deprecated TLS version
+            # or cipher is being used.
+
+            # `cipher()` returns a three-value tuple containing the name
+            # of the cipher being used, the version of the SSL protocol
+            # that defines its use, and the number of secret bits being used.
+            cipher, tls_version, _ = self._socket.sock.cipher()
+            warn_tls_version_deprecated(tls_version)
+            warn_ciphersuites_deprecated(cipher, tls_version)
 
     def shutdown(self) -> None:
         """Shut down connection to MySQL Server.
@@ -412,6 +428,7 @@ class MySQLConnection(MySQLConnectionAbstract):
     def close(self) -> None:
         """Disconnect from the MySQL server"""
         if self._span and self._span.is_recording():
+            # pylint: disable=possibly-used-before-assignment
             record_exception_event(self._span, sys.exc_info()[1])
 
         if not self._socket:
@@ -930,6 +947,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         ):
             raise ValueError("Invalid command REFRESH option")
 
+        res = None
         if options & RefreshOption.GRANT:
             res = self.cmd_query("FLUSH PRIVILEGES")
         if options & RefreshOption.LOG:
@@ -1023,12 +1041,13 @@ class MySQLConnection(MySQLConnectionAbstract):
         username: str = "",
         password: str = "",
         database: str = "",
-        charset: int = 45,
+        charset: Optional[int] = None,
         password1: str = "",
         password2: str = "",
         password3: str = "",
         oci_config_file: str = "",
         oci_config_profile: str = "",
+        openid_token_file: str = "",
     ) -> Optional[OkPacketType]:
         """Change the current logged in user
 
@@ -1037,10 +1056,14 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns a dict()
         """
-        if not isinstance(charset, int):
-            raise ValueError("charset must be an integer")
-        if charset < 0:
-            raise ValueError("charset should be either zero or a postive integer")
+        # If charset isn't defined, we use the same charset ID defined previously,
+        # otherwise, we run a verification and update the charset ID.
+        if charset is not None:
+            if not isinstance(charset, int):
+                raise ValueError("charset must be an integer")
+            if charset < 0:
+                raise ValueError("charset should be either zero or a postive integer")
+            self._charset_id = charset
 
         self._mfa_nfactor = 1
         self._user = username
@@ -1050,7 +1073,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._password3 = password3
 
         if self._password1 and password != self._password1:
-            password = self._password1
+            self._password = self._password1
 
         self.handle_unread_result()
 
@@ -1059,34 +1082,38 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         if oci_config_file:
             self._oci_config_file = oci_config_file
-
+        if openid_token_file:
+            self._openid_token_file = openid_token_file
         self._oci_config_profile = oci_config_profile
 
+        # Update the custom configurations needed by specific auth plugins
+        self._authenticator.update_plugin_config(
+            config={
+                "oci_config_file": self._oci_config_file,
+                "oci_config_profile": self._oci_config_profile,
+                "openid_token_file": self._openid_token_file,
+            }
+        )
+
         ok_pkt = self._authenticator.authenticate(
             sock=self._socket,
             handshake=self._handshake,
             username=self._user,
-            password1=password,
+            password1=self._password,
             password2=self._password2,
             password3=self._password3,
             database=database,
-            charset=charset,
+            charset=self._charset_id,
             client_flags=self._client_flags,
             auth_plugin=self._auth_plugin,
             auth_plugin_class=self._auth_plugin_class,
             conn_attrs=self._conn_attrs,
             is_change_user_request=True,
-            krb_service_principal=self._krb_service_principal,
-            oci_config_file=self._oci_config_file,
-            oci_config_profile=self._oci_config_profile,
-            webauthn_callback=self._webauthn_callback,
-            fido_callback=self._fido_callback,
         )
 
         if not (self._client_flags & ClientFlag.CONNECT_WITH_DB) and database:
             self.cmd_init_db(database)
 
-        self._charset_id = charset
         self._post_connection()
 
         # return ok_pkt
@@ -1160,6 +1187,7 @@ class MySQLConnection(MySQLConnectionAbstract):
                     self._password3,
                     self._oci_config_file,
                     self._oci_config_profile,
+                    self._openid_token_file,
                 )
             except ProgrammingError:
                 self.reconnect()
@@ -1222,10 +1250,10 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         It is possible to also give a custom cursor through the
         cursor_class parameter, but it needs to be a subclass of
-        mysql.connector.cursor.CursorBase.
+        mysql.connector.cursor.MySQLCursor.
 
         Raises ProgrammingError when cursor_class is not a subclass of
-        CursorBase. Raises ValueError when cursor is not available.
+        MySQLCursor. Raises ValueError when cursor is not available.
 
         Returns a cursor-object
         """
@@ -1234,9 +1262,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         if not self.is_connected():
             raise OperationalError("MySQL Connection not available")
         if cursor_class is not None:
-            if not issubclass(cursor_class, CursorBase):
+            if not issubclass(cursor_class, MySQLCursor):
                 raise ProgrammingError(
-                    "Cursor class needs be to subclass of cursor.CursorBase"
+                    "Cursor class needs be to subclass of MySQLCursor"
                 )
             return (cursor_class)(self)
 
@@ -1270,6 +1298,9 @@ class MySQLConnection(MySQLConnectionAbstract):
             24: MySQLCursorPreparedNamedTuple,
         }
         try:
+            raise_warning_against_deprecated_cursor_class(
+                cursor_name=types[cursor_type].__name__
+            )
             return (types[cursor_type])(self)
         except KeyError:
             args = ("buffered", "raw", "dictionary", "named_tuple", "prepared")
